/*
    Copyright (C) 2005-2008  Erik van Pienbroek

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <string.h>
#include <fcntl.h>
#include <errno.h>
#include <glib/gstdio.h>
#include <glib/gi18n.h>

#include "nntpgrab_plugin_decoder.h"
#include "yydecode.h"

// yydecode.c
int decode(const char *inname, const char *forced_outname, struct decoded_file **decoded_list);

static PluginDecoderImportedFuncs imported_funcs;

gboolean
nntpgrab_plugin_decoder_initialize(PluginDecoderImportedFuncs funcs)
{
    imported_funcs = funcs;

    return TRUE;
}

int
nntpgrab_plugin_decoder_get_version(void)
{
    return NNTPGRAB_PLUGIN_API_VERSION;
}

static void
add_line_to_message(char **msg, const char *line)
{
    char *tmp;

    if (*msg && strlen(*msg) > 0) {
        tmp = g_strdup_printf("%s\n%s", *msg, line);
        g_free(*msg);
    } else {
        tmp = g_strdup(line);
    }

    *msg = tmp;
}

gboolean
nntpgrab_plugin_decoder_decode_file(const char *collection_name, const NNTPFile *file, const char *temp_directory, const char *target_directory, int *saved_errno, char **errmsg)
{
    int i;
    int res;
    char *dir;
    GList *lines = NULL;
    gboolean ret = TRUE;
    gboolean file_found = FALSE;
    struct decoded_file *decoded_list = NULL;
    struct decoded_file *list;

    dir = g_strdup_printf("%s%s%s", target_directory, G_DIR_SEPARATOR_S, collection_name);
    if (g_mkdir_with_parents(dir, 0755) == -1) {
        g_print(_("Creation of folder '%s' failed, errno = %i, %s\n"), dir, errno, strerror(errno));
    }
    g_free(dir);

    dir = g_strdup_printf("%s%s%s%s", target_directory, G_DIR_SEPARATOR_S, collection_name, G_DIR_SEPARATOR_S);
    chdir(dir);
    g_free(dir);

    // Load all the individual parts
    for (i = 0; i < file->numparts; i++) {
        struct decoded_file *list;
        char *filename = g_strdup_printf("%s%s%s.%i", temp_directory, G_DIR_SEPARATOR_S, file->tmp_filename, i + 1);

        if (!g_file_test(filename, G_FILE_TEST_IS_REGULAR | G_FILE_TEST_EXISTS)) {
            g_free(filename);
            continue;
        }

        file_found = TRUE;

        if (decode(filename, NULL, &decoded_list) != EXIT_SUCCESS) {
            if (errmsg) {
                *errmsg = g_strdup(_("Unknown error occured during decode"));
            }
            g_print("Error during decode\n");
            ret = FALSE;
        }

        g_free(filename);
    }

    while (decoded_list) {
        list = decoded_list->next;
        free(decoded_list);
        decoded_list = list;
    }

    if (!file_found) {
        if (errmsg) {
            *errmsg = g_strdup(_("No data to decode"));
        }

        if (saved_errno) {
            *saved_errno = -1;
        }
    }

    if (lines && errmsg) {
        GList *list;
        char *line;

        *errmsg = NULL;

        if (g_list_length(lines) > 4) {
            // Take only the first two and the last two messages
            list = g_list_first(lines);
            line = list->data;
            add_line_to_message(errmsg, line);

            list = g_list_next(list);
            line = list->data;
            add_line_to_message(errmsg, line);

            list = g_list_last(lines);
            list = g_list_previous(list);
            line = list->data;
            add_line_to_message(errmsg, line);

            list = g_list_next(list);
            line = list->data;
            add_line_to_message(errmsg, line);

            add_line_to_message(errmsg, _("(not all output is shown)"));
        } else {
            list = lines;
            while (list) {
                line = list->data;
                add_line_to_message(errmsg, line);

                list = g_list_next(list);
            }
        }
    }

    if (!file_found || !lines) {
        // Remove all the temp parts on success
        for (i = 0; i < file->numparts; i++) {
            char *filename = g_strdup_printf("%s%s%s.%i", temp_directory, G_DIR_SEPARATOR_S, file->tmp_filename, i + 1);

            g_unlink(filename);

            g_free(filename);
        }
    }

    if (lines) {
        GList *list = lines;
        while (list) {
            g_free(list->data);
            list = g_list_next(list);
        }
        g_list_free(lines);
    }

    return ret;
}
